Package: piar
Title: Price Index Aggregation
Version: 0.10.1
Authors@R: c(
    person("Steve", "Martin", role = c("aut", "cre", "cph"),
           email = "marberts@protonmail.com",
           comment = c(ORCID = "0000-0003-2544-9480"))
    )
Description: Most price indexes are made with a two-step procedure, where
    period-over-period elementary indexes are first calculated for a collection
    of elementary aggregates at each point in time, and then aggregated according
    to a price index aggregation structure. These indexes can then be chained
    together to form a time series that gives the evolution of prices with
    respect to a fixed base period. This package contains a collection of
    functions that revolve around this work flow, making it easy to build
    standard price indexes, and implement the methods described by
    Balk (2008, <doi:10.1017/CBO9780511720758>), von der Lippe (2007,
    <doi:10.3726/978-3-653-01120-3>), and the CPI manual (2020,
    <doi:10.5089/9781484354841.069>) and PPI manual
    (2004, <doi:10.5089/9781589063044.069>) for bilateral price indexes.
Depends: R (>= 4.1)
Imports: stats, utils, Matrix (>= 1.5-0)
Suggests: data.tree, litedown, rsmatrix, spelling, sps, tinytest,
        treemap
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://marberts.github.io/piar/, https://github.com/marberts/piar
BugReports: https://github.com/marberts/piar/issues
LazyData: true
VignetteBuilder: litedown
Config/roxygen2/version: 8.0.0
Language: en-US
NeedsCompilation: no
Packaged: 2026-07-30 19:00:47 UTC; steve
Author: Steve Martin [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2544-9480>)
Maintainer: Steve Martin <marberts@protonmail.com>
Repository: CRAN
Date/Publication: 2026-07-30 23:30:02 UTC
Built: R 4.6.1; ; 2026-07-31 03:55:32 UTC; unix
