# Generated by roxygen2: do not edit by hand

S3method("[",piar_index)
S3method("[<-",chainable_piar_index)
S3method("[<-",direct_piar_index)
S3method("[<-",piar_index)
S3method("levels<-",piar_aggregation_structure)
S3method("levels<-",piar_index)
S3method("split<-",piar_index)
S3method("time<-",piar_index)
S3method("weights<-",piar_aggregation_structure)
S3method("window<-",piar_index)
S3method(Math,piar_index)
S3method(Ops,piar_index)
S3method(Summary,piar_index)
S3method(aggregate,chainable_piar_index)
S3method(aggregate,direct_piar_index)
S3method(anyNA,piar_index)
S3method(as.data.frame,piar_aggregation_structure)
S3method(as.data.frame,piar_index)
S3method(as.double,piar_index)
S3method(as.list,piar_aggregation_structure)
S3method(as.matrix,piar_aggregation_structure)
S3method(as.matrix,piar_index)
S3method(as.ts,piar_index)
S3method(as_aggregation_structure,data.frame)
S3method(as_aggregation_structure,default)
S3method(as_aggregation_structure,matrix)
S3method(as_aggregation_structure,piar_aggregation_structure)
S3method(as_index,chainable_piar_index)
S3method(as_index,data.frame)
S3method(as_index,default)
S3method(as_index,direct_piar_index)
S3method(as_index,matrix)
S3method(as_index,mts)
S3method(chain,chainable_piar_index)
S3method(chain,default)
S3method(chain,direct_piar_index)
S3method(cut,piar_aggregation_structure)
S3method(elementary_index,data.frame)
S3method(elementary_index,default)
S3method(elementary_index,numeric)
S3method(end,piar_index)
S3method(head,piar_index)
S3method(impute_prices,data.frame)
S3method(impute_prices,default)
S3method(impute_prices,matrix)
S3method(impute_prices,numeric)
S3method(is.na,piar_index)
S3method(levels,piar_aggregation_structure)
S3method(levels,piar_index)
S3method(mean,chainable_piar_index)
S3method(mean,direct_piar_index)
S3method(merge,chainable_piar_index)
S3method(merge,direct_piar_index)
S3method(merge,piar_index)
S3method(price_relative,data.frame)
S3method(price_relative,default)
S3method(print,piar_aggregation_structure)
S3method(print,piar_index)
S3method(rebase,chainable_piar_index)
S3method(rebase,default)
S3method(rebase,direct_piar_index)
S3method(split,piar_index)
S3method(stack,chainable_piar_index)
S3method(stack,direct_piar_index)
S3method(stack,piar_index)
S3method(start,piar_index)
S3method(str,piar_aggregation_structure)
S3method(str,piar_index)
S3method(summary,chainable_piar_index)
S3method(summary,direct_piar_index)
S3method(summary,piar_aggregation_structure)
S3method(tail,piar_index)
S3method(time,piar_index)
S3method(unchain,chainable_piar_index)
S3method(unchain,default)
S3method(unchain,direct_piar_index)
S3method(unstack,chainable_piar_index)
S3method(unstack,direct_piar_index)
S3method(unstack,piar_index)
S3method(update,piar_aggregation_structure)
S3method(weights,piar_aggregation_structure)
S3method(window,piar_index)
export("contrib<-")
export("time<-")
export("weights<-")
export(aggregation_structure)
export(as_aggregation_structure)
export(as_index)
export(back_period)
export(carry_backward)
export(carry_forward)
export(chain)
export(combine_classifications)
export(contrib)
export(contrib2DF)
export(elemental_index)
export(elementary_index)
export(emean)
export(expand_classification)
export(geks_index)
export(gmean)
export(hb_transform)
export(impute_prices)
export(interact_classifications)
export(is_aggregation_structure)
export(is_chainable_index)
export(is_direct_index)
export(is_index)
export(nested_gmean)
export(ntime)
export(outliers)
export(price_relative)
export(rebase)
export(scale_weights)
export(set_contrib)
export(set_contrib_from_index)
export(set_levels)
export(set_time)
export(set_weights)
export(shadow_price)
export(splice_index)
export(split_classification)
export(transmute_weights)
export(transmute_weights2)
export(unchain)
export(update_weights)
importFrom(stats,"window<-")
importFrom(stats,aggregate)
importFrom(stats,as.ts)
importFrom(stats,end)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,update)
importFrom(stats,weights)
importFrom(stats,window)
importFrom(utils,head)
importFrom(utils,stack)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,unstack)
