#' Price data
#'
#' Sample price and weight data for both a match sample and fixed sample type
#' index.
#'
#' @name price_data
#' @aliases ms_prices ms_weights fs_prices fs_weights
#' @docType data
NULL
